import test from "node:test";
import assert from "node:assert/strict";
import timers from "node:timers/promises";
import sql, { Database } from "@radically-straightforward/sqlite";
test(async () => {
    const database = new Database(":memory:");
    const migrations = [
        sql `
      CREATE TABLE "users" (
        "id" INTEGER PRIMARY KEY AUTOINCREMENT,
        "name" TEXT
      );
      CREATE TABLE "posts" (
        "id" INTEGER PRIMARY KEY AUTOINCREMENT,
        "content" TEXT,
        "author" INTEGER NOT NULL REFERENCES "users" ON DELETE CASCADE
      );
    `,
        async () => {
            database.execute(sql `
          INSERT INTO "users" ("name") VALUES (${"Leandro Facchinetti"});
        `);
        },
    ];
    await database.migrate(...migrations);
    assert.deepEqual(database.run(sql `
        INSERT INTO "users" ("name") VALUES (${"David Adler"})
      `), { changes: 1, lastInsertRowid: 2 });
    assert.deepEqual(database.get(sql `
        INSERT INTO "users" ("name") VALUES (${"Louie Renner"}) RETURNING *
      `), { id: 3, name: "Louie Renner" });
    assert.deepEqual(database.get(sql `
        SELECT "id", "name" FROM "users" WHERE "id" = 1
      `), { id: 1, name: "Leandro Facchinetti" });
    assert.equal(database.get(sql `
        SELECT "id", "name" FROM "users" WHERE "id" = 500
      `), undefined);
    assert.deepEqual(database.all(sql `
        SELECT "id", "name" FROM "users" ORDER BY "id" ASC
      `), [
        { id: 1, name: "Leandro Facchinetti" },
        { id: 2, name: "David Adler" },
        { id: 3, name: "Louie Renner" },
    ]);
    assert.deepEqual(database.all(sql `
        SELECT "id", "name" FROM "users" WHERE "name" IN ${[]}
      `), []);
    assert.deepEqual(database.all(sql `
        SELECT "id", "name" FROM "users" WHERE "name" IN ${[
        "Leandro Facchinetti",
        "David Adler",
    ]}
      `), [
        { id: 1, name: "Leandro Facchinetti" },
        { id: 2, name: "David Adler" },
    ]);
    assert.deepEqual(database.all(sql `
        SELECT "id", "name" FROM "users" WHERE "name" IN ${new Set([
        "Leandro Facchinetti",
        "David Adler",
    ])}
      `), [
        { id: 1, name: "Leandro Facchinetti" },
        { id: 2, name: "David Adler" },
    ]);
    assert.deepEqual([
        ...database.iterate(sql `
          SELECT "id", "name" FROM "users" ORDER BY "id" ASC
        `),
    ], [
        { id: 1, name: "Leandro Facchinetti" },
        { id: 2, name: "David Adler" },
        { id: 3, name: "Louie Renner" },
    ]);
    assert.equal(database.pragma("foreign_keys", { simple: true }), 1);
    assert.throws(() => {
        database.executeTransaction(() => {
            database.run(sql `
          INSERT INTO "users" ("name") VALUES (${"Abigail Wall"})
        `);
            throw new Error();
        });
    });
    assert.equal(database.get(sql `
        SELECT "id", "name" FROM "users" WHERE "name" = ${"Abigail Wall"}
      `), undefined);
    assert.deepEqual(database.executeTransaction(() => {
        return database.run(sql `
          INSERT INTO "users" ("name") VALUES (${"Abigail Wall"})
        `);
    }), { changes: 1, lastInsertRowid: 4 });
    assert.deepEqual(database.all(sql `
        SELECT "id", "name" FROM "users" WHERE "name" = ${"Abigail Wall"}
      `), [{ id: 4, name: "Abigail Wall" }]);
    let runsToCompletion = 0;
    migrations.push(() => {
        runsToCompletion++;
    });
    for (let iteration = 0; iteration < 5; iteration++) {
        await database.migrate(...migrations);
        assert.equal(runsToCompletion, 1);
    }
    await assert.rejects(async () => {
        await database.migrate(...migrations, sql `
        INSERT INTO "posts" ("content", "author")
        VALUES (${"Nonexistent author."}, 999999);
      `);
    });
    await assert.rejects(async () => {
        await database.migrate(...migrations, async () => {
            database.execute(sql `
          INSERT INTO "users" ("name") VALUES (${"Jeppe"})
        `);
            await timers.setTimeout();
            throw new Error("Rollback across ticks of the event loop.");
        });
    });
    assert.equal(database.get(sql `
        SELECT "id", "name" FROM "users" WHERE "name" = ${"Jeppe"}
      `), undefined);
    assert.deepEqual(sql `CREATE TABLE "users" ("id" INTEGER PRIMARY KEY AUTOINCREMENT, "name" TEXT)`, {
        sourceParts: [
            `CREATE TABLE "users" ("id" INTEGER PRIMARY KEY AUTOINCREMENT, "name" TEXT)`,
        ],
        parameters: [],
    });
    assert.deepEqual(sql `INSERT INTO "users" ("name") VALUES (${"Leandro Facchinetti"})`, {
        sourceParts: [`INSERT INTO "users" ("name") VALUES (`, `)`],
        parameters: ["Leandro Facchinetti"],
    });
    assert.deepEqual(sql `SELECT "id", "name" FROM "users" WHERE "name" IN ${[]}`, {
        sourceParts: [`SELECT "id", "name" FROM "users" WHERE "name" IN ()`],
        parameters: [],
    });
    assert.deepEqual(sql `SELECT "id", "name" FROM "users" WHERE "name" IN ${[
        "Leandro Facchinetti",
        "David Adler",
    ]}`, {
        sourceParts: [
            `SELECT "id", "name" FROM "users" WHERE "name" IN (`,
            `,`,
            `)`,
        ],
        parameters: ["Leandro Facchinetti", "David Adler"],
    });
    assert.deepEqual(sql `SELECT "id", "name" FROM "users" WHERE "name" IN ${new Set([])}`, {
        sourceParts: [`SELECT "id", "name" FROM "users" WHERE "name" IN ()`],
        parameters: [],
    });
    assert.deepEqual(sql `SELECT "id", "name" FROM "users" WHERE "name" IN ${new Set([
        "Leandro Facchinetti",
        "David Adler",
    ])}`, {
        sourceParts: [
            `SELECT "id", "name" FROM "users" WHERE "name" IN (`,
            `,`,
            `)`,
        ],
        parameters: ["Leandro Facchinetti", "David Adler"],
    });
    assert.deepEqual(sql `SELECT "id", "name" FROM "users" WHERE "name" = ${"Leandro Facchinetti"}$${sql ` AND "age" IS NOT NULL`}`, {
        sourceParts: [
            `SELECT "id", "name" FROM "users" WHERE "name" = `,
            ` AND "age" IS NOT NULL`,
        ],
        parameters: ["Leandro Facchinetti"],
    });
    assert.deepEqual(sql `SELECT "id", "name" FROM "users" WHERE "name" = ${"Leandro Facchinetti"}$${sql ` AND "age" = ${33}`}`, {
        sourceParts: [
            `SELECT "id", "name" FROM "users" WHERE "name" = `,
            ` AND "age" = `,
            ``,
        ],
        parameters: ["Leandro Facchinetti", 33],
    });
    assert.throws(() => {
        sql `SELECT "id", "name" FROM "users" WHERE "name" = ${"Leandro Facchinetti"}$${` AND "age" IS NOT NULL`}`;
    });
});
//# sourceMappingURL=index.test.mjs.map